#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Math::PlanePath::R5DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyR5Dragon ':all';
use MyLineSegments;
use MyPrintwrap ':all';


my $level = 3;

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";
my $path = Math::PlanePath::R5DragonCurve->new;

my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);

$Printwrap = 2;

my $segments = MyLineSegments->new;
foreach my $n ($n_lo .. $n_hi-1) {
  $segments->add($path->n_to_xy($n),
                 $path->n_to_xy($n+1));
}

my %count = (horizontal => 0,
             vertical   => 0);
foreach my $dir ('horizontal','vertical') {

  my $shorten = sub {
    my ($x1,$y1, $x2,$y2) = @_;
    if ($x1 == 0 && $y1 == 0) {
      return "start-$dir";
    }
    if ($x1 == $x_hi && $y1 == $y_hi) {
      return "end-$dir";
    }
    # if ($x1 == $x2) {
    #   my $sign = ($y1 < $y2 ? '+' : '-');
    #   $y1 = "$y1$sign\\MyShorten";
    # } else {
    #   my $sign = ($x1 < $x2 ? '+' : '-');
    #   $x1 = "$x1$sign\\MyShorten";
    # }
    return "$x1,$y1";
  };

  print "  \\begin{scope}[shift=(pos-$dir)]\n";
  if ($dir eq 'horizontal') {
    print "    \\node (start-$dir) at (0,0) [my dot];\n";
    print "    \\node (end-$dir) at ($x_hi,$y_hi) [my dot];\n";
  } else {
    print "    \\coordinate (start-$dir) at (0,0);\n";
    print "    \\coordinate (end-$dir) at ($x_hi,$y_hi);\n";
  }
  $Printwrap = 4;
  printwrap("\\draw");
  $Printwrap++;
  foreach my $line ($segments->lines) {
    my ($x1,$y1, $x2,$y2) = @$line;

    if ($x1 == $x2) {
      next unless $dir eq 'vertical';
    } else {
      next unless $dir eq 'horizontal';
    }
    $count{$dir}++;

    my $xy1 = $shorten->($x1,$y1, $x2,$y2);
    my $xy2 = $shorten->($x2,$y2, $x1,$y1);
    printwrap(" ($xy1)--($xy2)");
  }
  printwrap(";\n");
  print "  \\end{scope}\n";
}

my $Lines = $count{'horizontal'} + $count{'vertical'};
print "  \\newcommand{\\MyHorizontals}{$count{horizontal}}\n";
print "  \\newcommand{\\MyVerticals}{$count{vertical}}\n";
print "  \\newcommand{\\MyLines}{$Lines}\n";
print "% GP-Test-Last  Horizontals($level)==$count{horizontal}\n";
print "% GP-Test-Last  Verticals($level)==$count{vertical}\n";
print "% GP-Test-Last  Lines($level)==$Lines\n";

print "  % end generated\n";
exit 0;
