#!/usr/bin/perl -w

# Copyright 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;

use FindBin;
use lib $FindBin::Bin;
use MyR5Dragon;
use MyGraphs;

# uncomment this to run the ### lines
# use Smart::Comments;


my $max_level = 3;
if (@ARGV) { $max_level = shift @ARGV; }

print "% generated by tools/$FindBin::Script $max_level\n";

my $var = 'gen';

my %arrays;
foreach my $k (0 .. $max_level) {
  ### $k
  my $graph = MyR5Dragon::r5quad_area_tree_Graph($k);
  # MyGraphs::Graph_view($graph);
  my $apsp = $graph->APSP_Floyd_Warshall;
  my $QAdiameter = $apsp->diameter || 0;
  my $num_vertices = $graph->vertices;
  my %seen;
  my $QAdiameterCount = 0;
  my %QAdiameterEnds;
  my %QAdiameterVertices;
  if ($num_vertices == 1) {
    my @vertices = $graph->vertices;
    $QAdiameterEnds{$vertices[0]} = 1;
    $QAdiameterVertices{$vertices[0]} = 1;
  }
  $apsp->for_shortest_paths
    (sub {
       my ($t, $u,$v, $n) = @_;
       return if $seen{$u}->{$v};
       ### path: "$u to $v len ".$t->path_length($u,$v)
       if ($t->path_length($u,$v) == $QAdiameter) {
         $QAdiameterCount++;
         $seen{$u}->{$v} = 1;
         $seen{$v}->{$u} = 1;
         $QAdiameterEnds{$u} = 1;
         $QAdiameterEnds{$v} = 1;
         if ($k <= 3) {
           @QAdiameterVertices{$t->path_vertices($u,$v)} = ();  # hash slice
         }
       }
     });
  push @{$arrays{'QAdiameter'}},      $QAdiameter;
  push @{$arrays{'QAdiameterCount'}}, $QAdiameterCount;
  push @{$arrays{'QAdiameterEnds'}},  scalar(keys %QAdiameterEnds);
  if ($k <= 3) {
    push @{$arrays{'QAdiameterVertices'}},  scalar(keys %QAdiameterVertices);
  }
### %QAdiameterEnds
}

foreach my $name ('QAdiameter','QAdiameterCount',
                  'QAdiameterEnds','QAdiameterVertices',
                 ) {
  my $aref = $arrays{$name};
  print "% GP-DEFINE  ${var}_${name}_samples = [", join(',',@$aref),"];\n";
}

print "% end generated\n";
exit 0;
