#!/usr/bin/perl -w

# Copyright 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


# Wiener index and diameter of the R5 quad turns graph.

use 5.004;
use strict;

use FindBin;
use lib $FindBin::Bin;
use MyR5Dragon;
use MyGraphs;

# uncomment this to run the ### lines
# use Smart::Comments;


my $max_level = 3;
if (@ARGV) { $max_level = shift @ARGV; }

print "% generated by tools/$FindBin::Script $max_level\n";

my $var = 'gen';

my %prev_vertices;

my %arrays;
foreach my $k (0 .. $max_level) {
  ### $k
  my $graph = MyR5Dragon::r5quad_turn_Graph($k);
  # MyGraphs::Graph_view($graph);
  my @diameter = $graph->longest_path;
  my $QTdiameter = $graph->diameter || 0;

  # my @neighbours = (defined $diameter[0]
  #                   ? $graph->neighbours($diameter[0])
  #                   : undef);
  # my $Sv = $neighbours[0] // $diameter[0];

  my $Sv = '0,1';
  if ($k > 0) {
    $graph->has_vertex($Sv) or die;
  }

  my $QTW = 0;
  my $QTwC = 0;
  my $QTW_pairs = 0;
  my $QTdiameterCount = 0;

  my $for = $graph->for_shortest_paths
    (sub {
       my ($t, $u,$v, $n) = @_;
       my $path_length = $t->path_length($u,$v);
       if ($u lt $v) {  # one direction only
         $QTW += $path_length;
         $QTW_pairs++;

         if ($path_length == $QTdiameter) {
           $QTdiameterCount++;
         }
       }
       if ($u eq $Sv) {
         $QTwC += $path_length;
       }
     });

  push @{$arrays{'QTW'}},   $QTW;
  push @{$arrays{'QTwC'}},  $QTwC;
  push @{$arrays{'QTW_pairs'}}, $QTW_pairs;
  push @{$arrays{'QTdiameter'}}, $QTdiameter;
  push @{$arrays{'QTdiameterCount'}}, $QTdiameterCount;

  my @QTV = (0,0,0,0);
  foreach my $v ($graph->vertices) {
    $QTV[$graph->degree($v)]++;
  }
  $#QTV <= 3 or die;
  $QTV[0] == 0 or die;
  $QTV[1] == 0 or die;
  push @{$arrays{'QTV'}}, scalar($graph->vertices);
  push @{$arrays{'QTE'}}, scalar($graph->edges);
  push @{$arrays{'QTV2'}}, $QTV[2];
  push @{$arrays{'QTV3'}}, $QTV[3];

  if ($k > 0) {
    my $Cv;
    foreach my $v (keys %prev_vertices) {
      foreach my $neighbour ($graph->neighbours($v)) {
        if (! exists $prev_vertices{$neighbour}) {
          if (defined $Cv) {
            die "oops, already have Cv";
          }
          $Cv = $v;
        }
      }
    }
    push @{$arrays{'QTconn'}}, (defined $Cv ? $graph->path_length($Sv,$Cv) : 0);
  }

  %prev_vertices = ();
  @prev_vertices{$graph->vertices} = ();
}

foreach my $name ('QTdiameter','QTdiameterCount', 'QTconn',
                  'QTW',   'QTW_pairs',  'QTwC',
                  'QTV', 'QTV2', 'QTV3', 'QTE',
                 ) {
  my $aref = $arrays{$name} // [];
  print "% GP-DEFINE  ${var}_${name}_samples = [", join(',',@$aref),"];\n";
}

print "% end generated\n";
exit 0;
