#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017, 2018 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use Math::PlanePath::R5DragonCurve;

use FindBin;
use lib $FindBin::Bin;
use MyR5Dragon ':all';
use MyLineSegments;
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


print "  % generated by tools/$FindBin::Script\n";

my $segs = MyLineSegments->new;
foreach my $y (-1 .. 1) {
  foreach my $x (-1 .. 0) {
    $segs->add($x,$y, $x+1,$y);
  }
}
foreach my $y (-1 .. 0) {
  foreach my $x (-1 .. 1) {
    $segs->add($x,$y+1, $x,$y);
  }
}
# $segs->add(0,0, 1,0);
# $segs->add(1,0, 1,1);

$Printwrap = 2;
printwrap("\\node at (0,0) [my dot];\n");
printwrap("\\draw ");
$Printwrap = 4;
$segs->printwrap;
printwrap(";\n");

foreach my $level (1 .. 2) {
  my $new_segs = MyLineSegments->new;
  foreach my $line ($segs->segments) {
    my ($x1,$y1, $x2,$y2) = @$line;
    ### segment: "$x1,$y1 to $x2,$y2"

    my $dx = $x2 - $x1;
    my $dy = $y2 - $y1;
    # (x+iy)*(1+2i) = (x-2y) + (2x+y)i
    ($x1,$y1) = ($x1 - 2*$y1,
                 2*$x1 + $y1);
    ### multiply: "$x1,$y1"

    $new_segs->add($x1,$y1, $x1 + $dx, $y1 + $dy);
    $x1 += $dx; $y1 += $dy;

    $new_segs->add($x1,$y1, $x1 - $dy, $y1 + $dx);
    $x1 -= $dy; $y1 += $dx;

    $new_segs->add($x1,$y1, $x1 - $dx, $y1 - $dy);
    $x1 -= $dx; $y1 -= $dy;

    $new_segs->add($x1,$y1, $x1 - $dy, $y1 + $dx);
    $x1 -= $dy; $y1 += $dx;

    $new_segs->add($x1,$y1, $x1 + $dx, $y1 + $dy);
  }
  $segs = $new_segs;

  print "  \\begin{scope}[shift=(right-pos-$level)]\n";
  $Printwrap = 4;
  printwrap("\\node at (0,0) [my dot];\n");
  printwrap("\\draw ");
  $Printwrap += 2;
  $segs->printwrap;
  printwrap(";\n");
  print "  \\end{scope}\n";
}

print "  % end generated\n";
exit 0;
