#!/usr/bin/perl -w

# Copyright 2014, 2016, 2017, 2018, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use List::Util 'max';
use Math::PlanePath::R5DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyR5Dragon ':all';
use MyLineSegments;
use MyPrintwrap ':all';


my $level = 3;
my $chamfer = .25;

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

my $path = Math::PlanePath::R5DragonCurve->new;
my ($n_lo,$n_hi) = $path->level_to_n_range($level);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);

print "  \\begin{scope}[rotate={-atan2($y_hi,$x_hi)}]\n";

my $n_offset = 12*$n_hi;
my $n1 = $n_lo + 12*$n_hi;
my $n2 = 4*$n_hi + 12*$n_hi;
print "  % n_lo=$n_lo n_hi=$n_hi xy=$x_hi,$y_hi\n";

$Printwrap = 2;
for (my $n = $n1; $n < $n2; $n += $n_hi) {
  my ($x,$y) = $path->n_to_xy($n);
  printwrap(" \\node at ($x,$y) [my dot];");
}
printwrap_newline();

$Printwrap = 2;
printwrap("\\draw");
$Printwrap += 2;
my $join = '';

sub n_is_join {
  my ($n) = @_;
  if ($n % $n_hi == 0) {
    return 0;
  }
  my ($x,$y) = $path->n_to_xy($n);
  my @n_list = $path->xy_to_n_list($x,$y);
  if (@n_list < 2) {
    return 0;
  }
  foreach my $n (@n_list) {
    unless ($n >= $n1 && $n <= $n2) {
      return 0;
    }
  }
  my $np1 = int($n_list[0]/$n_hi);
  my $np2 = int($n_list[1]/$n_hi);
  return ($np1 != $np2);
}

my @squares;
my @squares_black;
my %seen;
foreach my $n ($n1 .. $n2-1) {
  my ($x,$y) = $path->n_to_xy($n);
  my ($dx,$dy) = $path->n_to_dxdy($n);

  if (n_is_join($n)) {
    my ($dx_prev,$dy_prev) = $path->n_to_dxdy($n-1);
    my $sx = $x - $chamfer*$dx_prev;
    my $sy = $y - $chamfer*$dy_prev;
    my $tx = $x + $chamfer*$dx;
    my $ty = $y + $chamfer*$dy;
    printwrap("$join($sx,$sy)--($tx,$ty)");
  } else {
    printwrap("$join($x,$y)");
  }

  if ($path->_UNDOCUMENTED__n_segment_is_right_boundary($n % $n_hi)) {

    my $x1 = $x;
    my $y1 = $y;
    my $x2 = $x1 + $dx + $dy;  # opposite corner
    my $y2 = $y1 + $dy - $dx;

    ($x1,$y1, $x2,$y2) = rect_corners_sort ($x1,$y1, $x2,$y2);
    next if $seen{"$x1,$y1"}++;

    my $middle = ($x1 == -2 && $y1 == -11) || ($x1 == 19 && $y1 == -14);
    if ($middle) {
      push @squares_black, "($x1,$y1),";
    } else {
      push @squares, "($x1,$y1),";
    }
  }

  $join = '--';
}
printwrap("--cycle;\n");

$Printwrap = 2;
printwrap("\\fill foreach \\p in {");
$Printwrap += 2;
foreach my $square (@squares_black) {
  printwrap($square);
}
printwrap("}");
printwrap("{ \\p +(\\MyMargin,\\MyMargin) rectangle +(1-\\MyMargin,1-\\MyMargin) };\n");

$Printwrap = 2;
printwrap("\\fill[my grey] foreach \\p in {");
$Printwrap += 2;
foreach my $square (@squares) {
  printwrap($square);
}
printwrap("}");
printwrap("{ \\p +(\\MyMargin,\\MyMargin) rectangle +(1-\\MyMargin,1-\\MyMargin) };\n");

print "  \\end{scope}\n";
print "  % end generated\n";
exit 0;
