#!/usr/bin/perl -w

# Copyright 2017, 2018 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use FindBin;
use Graph::Maker::QuartetTree;
use List::Util 'min';

use lib $FindBin::Bin;
use MyR5Dragon 'rect_corners_sort';
use MyLineSegments;
use MyPrintwrap ':all';


my $level = 3;
if (@ARGV) { $level = shift @ARGV; }

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

# return $x,$y multiplied by 2+i
# GP-Test  (2+I)*(x+I*y) == (2*x-y) + (x+2*y)*I
sub mul {
  my ($x,$y) = @_;
  return (2*$x -   $y,
          $x   + 2*$y);
}

my $ex = 1;
my $ey = 0;

my $graph = Graph::Maker->new('quartet_tree',
                              level => $level,
                              undirected=>1);
my (@horizontal, @vertical);
my @edges;
sub add_edge {
  my ($from,$to) = @_;
  my ($x1,$y1) = split /,/,$from;
  my ($x2,$y2) = split /,/,$to;
  ($x1,$y1, $x2,$y2) = rect_corners_sort ($x1,$y1, $x2,$y2);
  if ($y1 == $y2) {
    $x1 .= '+\MyRadius';
    $x2 .= '-\MyRadius';
  } else {
    $y1 .= '+\MyRadius';
    $y2 .= '-\MyRadius';
  }
  push @edges, " ($x1,$y1)--($x2,$y2)";
}
while ($graph->vertices) {
  foreach my $from ($graph->vertices) {
    next unless $graph->has_vertex($from);
    next unless $graph->vertex_degree($from) == 1;
    my ($to) = $graph->neighbours($from);
    my ($x1,$y1) = split /,/,$from;
    my ($x2,$y2) = split /,/,$to;
    if ($y1 == $y2) {
      my $x = min($x1,$x2);
      push @horizontal, "$x,$y1";
    } else {
      my $y = min($y1,$y2);
      push @vertical, "$x1,$y";
    }
    $graph->delete_edge($from,$to);
    foreach my $other ($graph->neighbours($from)) {
      add_edge($from, $other);
    }
    foreach my $other ($graph->neighbours($to)) {
      add_edge($to, $other);
    }
    $graph->delete_vertices($from,$to);
  }
}

$Printwrap = 2;
printwrap("\\path foreach \\p in {");
$Printwrap += 2;
foreach my $xy (sort @horizontal) {
  printwrap("($xy),");
}
printwrap("}");
printwrap(" { \\p pic[transform shape] {my match pic} };\n");

$Printwrap = 2;
printwrap("\\path foreach \\p in {");
$Printwrap += 2;
foreach my $xy (sort @vertical) {
  printwrap("($xy),");
}
printwrap("}");
printwrap(" { \\p pic[transform shape,rotate=90] {my match pic} };\n");

$Printwrap = 2;
printwrap("\\draw[my edge]");
$Printwrap += 2;
foreach my $edge (sort @edges) {
  printwrap($edge);
}
printwrap(";\n");

print "  % end generated\n";
exit 0;
