#!/usr/bin/perl -w

# Copyright 2017, 2018 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


# Drawing downwards from the diameter, sub-trees directed left and down to
# make room for caption at right.

use 5.004;
use strict;
use FindBin;
use Graph::Maker::QuartetTree;
use List::Util 'min','max';
use Tie::IxHash;

use lib $FindBin::Bin;
use MyR5Dragon 'xy_to_node_name';
use MyLineSegments;
use MyPrintwrap ':all';


# uncomment this to run the ### lines
# use Smart::Comments;


my $level = 6;
my $max_depth = 42;
# WidthS_infinity_k($max_depth+1) <= $level or die;
# RQ($level-1) > $max_depth or die;

print "  % generated by tools/$FindBin::Script\n";

# $v1 and $v2 are vertices of $graph and $graph is a tree.
# $v1 is the parent of $v2
# Return the height (longest path to leaf) of the subtree at $v2.
sub Graph_subtree_height {
  my ($graph, $v1, $v2) = @_;
  ### $v1
  ### $v2
  $graph->has_edge($v1,$v2) or die "oops, $v1 and $v2 not adjacent";
  my $depth = 0;
  my %seen = ($v1 => 1, $v2 => 1);
  my @pending = ($v2);
  do {
    @pending = map {$graph->neighbours($_)} @pending;
    @pending = grep {! $seen{$_}++} @pending;
    $depth++;
  } while (@pending);
  return $depth;
}

my $graph = Graph::Maker->new('quartet_tree', level=>$level,
                              undirected=>1);
my $root = '0,0';

my @vertices = ($root);
my %seen; tie %seen, 'Tie::IxHash';
my $x = 0;
my @widths = (1);
my %vertex_to_location = ($root => [0,0]);
$seen{$root} = 1;
my %nodes; tie %nodes, 'Tie::IxHash';
my @segments;
while (@vertices) {
  if (++$x > $max_depth) { last; }
  my $width = 0;
  my $y = 0;
  my @new_vertices;
  foreach my $v (@vertices) {
    my $v_location = $vertex_to_location{$v};
    my ($vx,$vy) = @$v_location;
    my @children = $graph->neighbours($v);
    ### $v
    ### neighbours: @children
    @children = grep {! $seen{$_}} @children;
    @children = sort
      {Graph_subtree_height($graph,$v,$b)<=>Graph_subtree_height($graph,$v,$a)}
      @children;
    ### children: @children
    foreach my $child (@children) {
      $y = min($y, $vy);
      $vertex_to_location{$child} = [$x,$y];

      my $from = xy_to_node_name($vx,$vy);
      my $to   = xy_to_node_name($x,$y);
      push @segments, " ($from)--($to)";
      $nodes{$from} = "$vx,$vy";
      $nodes{$to}   = "$x,$y";

      push @new_vertices, $child;
      $seen{$child} = 1;
      $y--;
      $width++;
    }
  }
  push @widths, $width;
  @vertices = @new_vertices;
}

$Printwrap = 2;
while (my ($name, $xy) = each %nodes) {
  printwrap(" \\node ($name) at ($xy) [my dot];");
}
$Printwrap = 2;
printwrap("\\draw");
$Printwrap += 2;
foreach my $segment (@segments) {
  printwrap($segment);
}
printwrap(";\n");

{
  # dashed lines at end
  $Printwrap = 2;
  printwrap("\\draw[my dashed]");
  $Printwrap = 4;
  foreach my $v (@vertices) {
    my @children = $graph->neighbours($v);
    @children = grep {! $seen{$_}} @children;
    if (@children) {
      my $p = $vertex_to_location{$v};
      my ($x,$y) = @$p;
      my $name = xy_to_node_name($x,$y);
      printwrap(" ($name)--+(\\MyDash,0)");
    }
  }
  printwrap(";\n");
}

# {
#   # dots
#   $Printwrap = 2;
#   printwrap("\\path foreach \\p in {");
#   $Printwrap = 4;
#   foreach my $v (keys %seen) {
#     my $p = $vertex_to_location{$v};
#     my ($x,$y) = @$p;
#     printwrap("($x,$y),");
#   }
#   printwrap("} ");
#   printwrap("{ \\p node[my dot] };\n");
# }

print "% GP-DEFINE  QuartetWidth_infinity_generated_samples = [",
  join(',',@widths),"];\n";
print "  % end generated\n";
exit 0;
