#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017, 2020 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Math::BaseCnv 'cnv';
use Math::PlanePath::R5DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyR5Dragon 'dir';
use MyPrintwrap ':all';


my $level = 2;
my $chamfer = .15;
print "    % generated by tools/$FindBin::Script\n";

my $path = Math::PlanePath::R5DragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);

my @dir4_to_label_side = ('below','right','above','left');
sub n_to_label_side {
  my ($n) = @_;
  return $dir4_to_label_side[dir($n) % 4];
}

my @n_to_boundary_type = (['R',     # none
                           'X'],    # post only
                          ['Z',     # pre only
                           'Y']);   # both
sub n_to_boundary_type {
  my ($n) = @_;

  my ($x,$y) = $path->n_to_xy($n);
  my ($dx,$dy) = $path->n_to_dxdy($n);
  ($dx,$dy) = ($dy,-$dx);  # rotate -90
  my $pre = xyxy_is_traversed($x,$y, $x+$dx,$y+$dy);

  ($x,$y) = $path->n_to_xy($n+1);
  my $post = xyxy_is_traversed($x,$y, $x+$dx,$y+$dy);

  return $n_to_boundary_type[$pre][$post];
}
sub xyxy_is_traversed {
  my ($x1,$y1, $x2,$y2) = @_;
  my $n = $path->xyxy_to_n_either($x1,$y1, $x2,$y2);
  return (defined $n && $n < $n_hi);
}

sub n_to_point {
  my ($n) = @_;
  if ($n % 5 == 0) {
    $n /= 5;
    return "(p$n)";
  }
  my ($x,$y) = $path->n_to_xy($n);
  my @n_list = $path->xy_to_n_list($x,$y);
  if (@n_list == 1) {
    return "($x,$y)";
  }

  my ($dx,$dy) = $path->n_to_dxdy($n-1);
  my $sx = $x - $dx*$chamfer;
  my $sy = $y - $dy*$chamfer;
  ($dx,$dy) = $path->n_to_dxdy($n);
  my $tx = $x + $dx*$chamfer;
  my $ty = $y + $dy*$chamfer;
  return "($sx,$sy)", "($tx,$ty)";
}


my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);

$Printwrap = 4;
foreach my $n (0 .. 5) {
  my ($x,$y) = $path->n_to_xy(5*$n);
  printwrap("\\node (p$n) at ($x,$y) [circle,draw,inner sep=.1em] {$n};\n");
}

printwrap("    \\draw ");
my $join = '';
$Printwrap = 6;
foreach my $n ($n_lo .. $n_hi) {
  my ($p1,$p2) = n_to_point($n);

  my $str = "$join$p1";
  $join = '--';

  if ($n > $n_lo && $path->_UNDOCUMENTED__n_segment_is_right_boundary($n-1)) {
    my $type = n_to_boundary_type($n-1);
    my $side = n_to_label_side($n-1);
    $str .= "node[$side,midway]{$type}";
    $join = ' --';
  }
  printwrap($str);

  if ($p2) {
    printwrap("$join$p2");
    $join = '--';
  }
}
printwrap(";\n");

print "    % end generated\n";
exit 0;
