#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use Math::PlanePath::R5DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyR5Dragon;
use MyLineSegments;
use MyPrintwrap ':all';

my $level = 4;
if (@ARGV) { $level = shift @ARGV; }

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

my $path = Math::PlanePath::R5DragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);

# print "  \\begin{scope}[rotate={-atan2($y_hi,$x_hi)},scale={1/sqrt(5)^$level}]\n";
print "  \\coordinate (start) at (0,0);\n";
print "  \\coordinate (end) at ($x_hi,$y_hi);\n";
print "  \\node (start-dot) at (start) [my small dot];\n";
print "  \\node (end-dot) at (end) [my small dot];\n";

my $segments = MyLineSegments->new;
foreach my $n ($n_lo .. $n_hi-1) {
  $segments->add($path->n_to_xy($n),
                 $path->n_to_xy($n+1));
}

{
  $Printwrap = 2;
  printwrap("\\draw ");
  $Printwrap += 2;
  $segments->printwrap (stringize => sub {
                          my ($x,$y) = @_;
                          if ($x==0 && $y==0) { return '(start-dot)'; }
                          if ($x==$x_hi && $y==$y_hi) { return '(end-dot)'; }
                          return undef;
                        });
  printwrap(";");
  my $count = $segments->lines_joined_count_points;
  printwrap(" % $count points\n");
}

# print "  \\end{scope}\n";
print "  % end generated\n";
exit 0;
