# Copyright 2015 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  See the file COPYING.  If not, see
# <http://www.gnu.org/licenses/>.


package XYconvex;
use 5.004;
use strict;
use Carp 'croak';
use List::Util 'min','max';

# uncomment this to run the ### lines
# use Smart::Comments;

sub new {
  my $class = shift;
  return bless { rows_x_min => [],
                 rows_x_max => [],
                 cols_y_min => [],
                 cols_y_max => [],
                 rows_offset => undef,
                 cols_offset => undef,

               }, $class;
}
sub bbox {
  my ($self) = @_;
  ### bbox(): $self
  return ($self->x_minimum,
          $self->y_minimum,
          $self->x_maximum,
          $self->y_maximum);
}
sub x_minimum {
  my ($self) = @_;
  return $self->{'cols_offset'};
}
sub x_maximum {
  my ($self) = @_;
  return $self->{'cols_offset'}
    && ($self->{'cols_offset'} + $#{$self->{'cols_y_min'});
}
sub y_minimum {
  my ($self) = @_;
  return $self->{'rows_offset'};
}
sub y_maximum {
  my ($self) = @_;
  return $self->{'rows_offset'}
    && ($self->{'rows_offset'} + $#{$self->{'rows_x_min'});
}
sub dummy {}

sub add_point {
  my ($self, $x,$y) = @_;
  my @pending = ($x,$y);
  while (@pending) {
    while (@pending) {
      my $y = pop @pending;
      my $x = pop @pending;
      _add_point_raw($self, $x,$y);
    }
    @pending = _infill($self);
  }
}

sub _infill {
  my ($self) = @_;
  my @ret;

  {
    my $max;
    my $x_max;
    foreach my $x ($self->x_minimum .. $self->y_minimum) {
    }
  }
}

sub add_point_raw {
  my ($self, $x,$y) = @_;
  if (! defined $self->{'rows_offset'}) {
    $self->{'rows_offset'} = $x;
    $self->{'cols_offset'} = $y;
    $self->{'rows_x_min'}->[$y - $self->{'rows_offset'}] = $x;
    $self->{'rows_x_max'}->[$y - $self->{'rows_offset'}] = $x;
    $self->{'cols_y_min'}->[$x - $self->{'cols_offset'}] = $y;
    $self->{'cols_y_max'}->[$x - $self->{'cols_offset'}] = $y;
    return;
  }

  if ((my $extra = $self->x_minimum - $x) > 0) {
    $self->{'cols_offset'} = $x;
    foreach my $i (0 .. $extra-1) {
      $self->{'cols_y_min'}->[$i] = $y;
      $self->{'cols_y_max'}->[$i] = $y;
    }
  } elsif ((my $extra = $x - $self->x_maximum) > 0) {
    foreach (0 .. $extra-1) {
      push @{$self->{'cols_y_min'}}, $y;
      push @{$self->{'cols_y_max'}}, $y;
    }
  }
    
  if ((my $extra = $self->y_minimum - $y) > 0) {
    $self->{'rows_offset'} = $y;
    foreach my $i (0 .. $extra-1) {
      $self->{'rows_x_min'}->[$i] = $x;
      $self->{'rows_x_max'}->[$i] = $x;
    }
  } elsif ((my $extra = $y - $self->y_maximum) > 0) {
    foreach (0 .. $extra-1) {
      push @{$self->{'rows_x_min'}}, $x;
      push @{$self->{'rows_x_max'}}, $x;
    }
  }
    
  $self->{'rows_x_min'}->[$y - $self->{'rows_offset'}]
    = min($self->{'rows_x_min'}->[$y - $self->{'rows_offset'}],
          $x);

  $self->{'rows_x_max'}->[$y - $self->{'rows_offset'}]
    = max($self->{'rows_x_max'}->[$y - $self->{'rows_offset'}]
          $x);

  $self->{'cols_y_min'}->[$x - $self->{'cols_offset'}]
    = min($self->{'cols_y_min'}->[$x - $self->{'cols_offset'}]
          $y);

  $self->{'cols_y_max'}->[$x - $self->{'cols_offset'}]
    = max($self->{'cols_y_max'}->[$x - $self->{'cols_offset'}],
          $y);
  
}
1;
__END__
