#!/usr/bin/perl -w

# Copyright 2016, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use List::Util 'min';
use Math::PlanePath::R5DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyR5Dragon '$hullsaround_segs_outer','$hullsaround_segs_inner',
  '$hullsaround_segs_all', 'xy_to_node_name', '@dir4_to_dx','@dir4_to_dy',
  'xy_rotate_plus90','rect_corners_sort';
use MyLineSegments;
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;

print "  % generated by tools/$FindBin::Script\n";


$Printwrap = 2;

{
  my @squares;
  foreach my $side (0, 1) {
    foreach my $seg (sort $hullsaround_segs_all->segs_list) {
      my ($x,$y,$dir) = split /,/, $seg;
      my $dx = $dir4_to_dx[$dir];
      my $dy = $dir4_to_dy[$dir];
      my $ddir = $side==0 ? -1 : 1;
      my $ex = $x + $dx;
      my $ey = $y + $dy;
      my $oseg = "$ex,$ey,".($dir+$ddir);
      next unless $hullsaround_segs_all->contains($oseg);
      my ($sx,$sy) = xy_rotate_plus90($dx,$dy, $ddir);
      my $ox = $ex + $sx;
      my $oy = $ey + $sy;
      my ($cx,$cy) = rect_corners_sort ($x,$y, $ox,$oy);
      $squares[$side]->{"$cx,$cy"} = 1;
      ### seg: "$seg ddir=$ddir end $ex,$ey to oseg=$oseg   sxy=$sx,$sy for rect $x,$y $ox,$y"
      ### corner: "$cx,$cy side $side"
    }
  }
  ### @squares

  # left fills
  $Printwrap = 2;
  printwrap("\\fill[my L colour] foreach \\p in {");
  $Printwrap += 2;
  foreach my $xy ( keys %{$squares[1]}) { printwrap("($xy),"); }
  printwrap("}");
  printwrap("{ \\p +(border) rectangle +(\$(1,1)-(border)\$) };\n");

  # R, L text
  my @RL = ('R','L');
  print "  \\begin{scope}[font=\\scriptsize]\n";
  $Printwrap = 2;
  foreach my $side (0, 1) {
    foreach my $xy (sort keys %{$squares[$side]}) {
      printwrap(" \\node at (\$($xy)+(.5,.5)\$) {$RL[$side]};");
    }
  }
  printwrap_newline();
  print "  \\end{scope}\n";
}

{
  $Printwrap = 2;
  my %seen;
  foreach my $seg (sort $hullsaround_segs_all->segs_list) {
    my ($x,$y,$dir) = split /,/, $seg;
    foreach (0, 1) {
      my $name = xy_to_node_name($x,$y);
      unless ($seen{$name}++) {
        my $blank = ($x==0 && $y==0) || ($x==1 && $y==0) ? '' : ' blank';
        printwrap(" \\node ($name) at ($x,$y) [my$blank dot];");
      }
      $x += $dir4_to_dx[$dir];
      $y += $dir4_to_dy[$dir];
    }
  }
  printwrap_newline();
}

$Printwrap = 2;
foreach my $seg (sort $hullsaround_segs_all->segs_list) {
  next if $seg eq '0,0,0';
  my ($x,$y,$dir) = split /,/, $seg;
  my $from = xy_to_node_name($x,$y);
  my $to   = xy_to_node_name($x+$dir4_to_dx[$dir], $y+$dir4_to_dy[$dir]);
  printwrap(" \\draw[->] ($from)--($to);");
}
printwrap_newline();

print "  % end generated\n";
exit 0;
